﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data;
using System.Data.SqlClient;
using System.IO;
using System.Runtime.InteropServices;
using Microsoft.Win32.SafeHandles;

namespace FilestreamWriter
{
    class Program
    {
        [DllImport("sqlncli10.dll", SetLastError = true, CharSet = CharSet.Unicode)]
        public static extern SafeFileHandle OpenSqlFilestream(
                    string FilestreamPath,
                    UInt32 DesiredAccess,
                    UInt32 OpenOptions,
                    byte[] FilestreamTransactionContext,
                    UInt32 FilestreamTransactionContextLength,
                    LARGE_INTEGER_SQL AllocationSize);

        [StructLayout(LayoutKind.Sequential)]
        public struct LARGE_INTEGER_SQL
        {
            public Int64 QuadPart;
            public LARGE_INTEGER_SQL(Int64 quadPart) { QuadPart = quadPart; }
        }

        //Get Last Error to check if the OpenSqlFilestream succeeded in returning a valid / handle
        //
        [DllImport("kernel32.dll", SetLastError = true)]
        public static extern UInt32 GetLastError();

        // DesiredAccess
        //
        const UInt32 DESIRED_ACCESS_READ = 0x00000000;
        const UInt32 DESIRED_ACCESS_WRITE = 0x00000001;
        const UInt32 DESIRED_ACCESS_READWRITE = 0x00000002;

        static void Main(string[] args)
        {
            string connectionString = "server=.;integrated security=sspi;database=filestreamdb";

            string filePath = string.Empty;
            SqlConnection sqlConnection = new SqlConnection(connectionString);
            sqlConnection.Open();

            SqlTransaction tx = sqlConnection.BeginTransaction();

            SqlCommand cmd = new SqlCommand();
            cmd.CommandText = "SELECT TOP(1) Resume.PathName() FROM dbo.student where name = 'Fred'";
            cmd.CommandType = CommandType.Text;
            cmd.Connection = sqlConnection;
            cmd.Transaction = tx;

            Object pathObj = cmd.ExecuteScalar();
            if (DBNull.Value != pathObj)
            {
                filePath = (string)pathObj;
            }

            byte[] transactionToken;

            cmd.CommandText = "SELECT GET_FILESTREAM_TRANSACTION_CONTEXT()";
            cmd.CommandType = CommandType.Text;
            cmd.Connection = sqlConnection;

            Object txObj = cmd.ExecuteScalar();
            if (DBNull.Value != txObj)
            {
                transactionToken = (byte[])txObj;
            }
            else
            {
                transactionToken = null;
                // bail, an error
            }

            // The following code demonstrates how to write to the file
            SafeFileHandle handle = OpenSqlFilestream(
                         filePath,
                         DESIRED_ACCESS_WRITE,
                         0,
                         transactionToken,
                         (UInt32)transactionToken.Length,
                         new LARGE_INTEGER_SQL(0));

            UInt32 NtStatus = GetLastError();

            // buffer-worth
            // this must be filled with data to write
            byte[] contentToWrite = new byte[4096];
            contentToWrite[0] = (byte)'1';
            contentToWrite[1] = (byte)'2';
            contentToWrite[2] = (byte)'3';

            FileStream fs = new FileStream (handle, 
                                  FileAccess.Write, 
                                  contentToWrite.Length, 
                                  false);

            fs.Write(contentToWrite, 0, contentToWrite.Length);

            fs.Close();

            if (handle != null && !handle.IsClosed)
            {
                Console.WriteLine("\n\nClosing handle");
                handle.Close();
            }

            tx.Commit();
        }
    }
}
