/*============================================================================
  File:     filestream_northwind.sql

  Summary:  This script demonstrates redefining a table with currently uses
			the IMAGE data type to use a VARBINARY(MAX) FILESTREAM data type.
			The actual pictures are then stored on the file system.
			
			This script also serves as a "setup script" for the ADO.NET 
			demo program FileStreamForms and other Filestream access 
			programs in this directory.

  Date:     August 2008

  SQL Server Version: 10.0.1600.22 (RTM)
------------------------------------------------------------------------------
  Written by Bob Beauchemin, SQLskills.com

  For more scripts and sample code, check out http://www.SQLskills.com
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

use Northwind
go

-- this is to enable at an engine level
-- NOTE: Filestream must also be configured at an OS level using 
-- SQL Server Configuration Manager (see BOL for details)
EXEC sp_configure 'filestream access level', '2'
RECONFIGURE
GO

ALTER DATABASE Northwind
 ADD FILEGROUP FileStreamGroup1 CONTAINS FILESTREAM
GO

ALTER DATABASE Northwind
    ADD FILE
      (
       NAME = Photos,
       FILENAME = 'C:\Files\Photos'
      )
   TO FILEGROUP FileStreamGroup1
GO

USE Northwind
GO

CREATE TABLE [dbo].[Employees2](
	[EmployeeID] [int] IDENTITY(1,1) NOT NULL,
	[LastName] [nvarchar](20) NOT NULL,
	[FirstName] [nvarchar](10) NOT NULL,
	[Title] [nvarchar](30) NULL,
	[Photo] [varbinary](max) FILESTREAM NULL,
	[ReportsTo] [int] NULL,
	[RowGuid] [UNIQUEIDENTIFIER]  NOT NULL  ROWGUIDCOL UNIQUE DEFAULT NEWID(),
 CONSTRAINT [PK_Employees2] PRIMARY KEY CLUSTERED 
(
	[EmployeeID] ASC
)WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY] 

GO

ALTER TABLE [dbo].[Employees2]  WITH NOCHECK ADD  CONSTRAINT [FK_Employees2_Employees2] FOREIGN KEY([ReportsTo])
REFERENCES [dbo].[Employees2] ([EmployeeID])
GO

ALTER TABLE [dbo].[Employees2] CHECK CONSTRAINT [FK_Employees2_Employees2]
GO

SET IDENTITY_INSERT Employees2 ON
GO

INSERT INTO [dbo].[Employees2] (EmployeeID, LastName, FirstName, Title, Photo, ReportsTo)
SELECT EmployeeID, LastName, FirstName, Title, CONVERT(VARBINARY(MAX),Photo), ReportsTo
FROM Employees
GO

-- now add an attribute of the file as a persisted computed column
ALTER TABLE [dbo].[Employees2]
  ADD Photo_Width AS SUBSTRING(Photo, 19,4) PERSISTED 
  
ALTER TABLE [dbo].[Employees2]
  ADD Photo_Height AS SUBSTRING(Photo, 23,4) PERSISTED 
  
-- No index allowed on the persisted computed column, though, this fails
-- CREATE INDEX IX_PhotoWidth ON Employees2(Photo_Width)
GO

SELECT photo.PathName(),* FROM [dbo].[Employees2]

!! dir c:\files\Photos /s

-- or alter the table and add filestream column
ALTER TABLE Employees SET (FILESTREAM_ON = "default")
GO

-- add filestream and rowguid columns
ALTER TABLE Employees ADD
	[RowGuid] [UNIQUEIDENTIFIER]  NOT NULL  ROWGUIDCOL UNIQUE DEFAULT NEWID(),
	[Photo2] [varbinary](max) FILESTREAM NULL
GO

-- populate the new column
UPDATE Employees SET Photo2 = Photo
GO

ALTER TABLE Employees DROP COLUMN Photo
GO

-- rename the new column to the original name
EXEC dbo.sp_rename @objname=N'[dbo].[Employees].[Photo2]', 
                   @newname=N'Photo', 
                   @objtype=N'COLUMN'
GO
	
	